/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace;

import java.util.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

/**
 * Histogram
 * 
 * @author Christian
 * @author Thomas
 * @author Andreas
 */
@SuppressWarnings("serial")
public class Histogram extends JPanel {
	private boolean showMeterBars = true;
	private boolean showRSSCounts = true;
	private int maxValue = 20;
	private int oldMaxValue = 20;
	private int yStep = new Double(Math.floor(350 / maxValue)).intValue();
	private ArrayList<Hashtable<String, Integer>> history = new ArrayList<Hashtable<String, Integer>>();
	private ArrayList<String> apList = new ArrayList<String>();
	private ArrayList<Color> colorList = new ArrayList<Color>(16);
	private Hashtable<String, Integer> rssCount = new Hashtable<String, Integer>();
	
	public Histogram() {
		// colors
		colorList.add(new Color(255, 0, 0));
		colorList.add(new Color(0, 255, 0));
		colorList.add(new Color(0, 0, 255));
		colorList.add(new Color(255, 0, 255));
		colorList.add(new Color(0, 255, 255));
		colorList.add(new Color(123, 104, 238));
		colorList.add(new Color(255, 140, 0));
		colorList.add(new Color(205, 104, 57));
		colorList.add(new Color(255, 128, 0));
		colorList.add(new Color(128, 255, 0));
		colorList.add(new Color(128, 0, 255));
		colorList.add(new Color(205, 0, 205));
		colorList.add(new Color(0, 205, 205));
		colorList.add(new Color(73, 54, 188));
		colorList.add(new Color(205, 90, 0));
		colorList.add(new Color(155, 54, 7));
		colorList.add(new Color(128, 0, 0));
		colorList.add(new Color(0, 128, 0));
		colorList.add(new Color(0, 0, 128));
		colorList.add(new Color(64, 0, 0));
		colorList.add(new Color(0, 64, 0));
		colorList.add(new Color(0, 0, 64));
		
		setMinimumSize(new Dimension(1000, 450));
	}
	
	private static final int minXValue = -127, maxXValue = 20;
	
	protected void paintComponent(Graphics g) {
		g.setColor(Color.BLACK);
		g.fillRect(0, 0, 1000, 730);
		// draw dividing lines
		g.setColor(Color.WHITE);
		g.drawLine(0, 360, 999, 360);
		g.drawLine(164, 0, 164, 729);
		// draw x scale
		int offsetX = 164;
		int xStep = (980 - offsetX) / (maxXValue - minXValue);
		for (int i = minXValue; i <= maxXValue; i++) {
			if ((i % 10) == 0) {
				g.drawLine(offsetX + (i-minXValue)*xStep, 356, offsetX + (i-minXValue)*xStep, 364);
				String s = Integer.toString(i);
				Rectangle2D rect = g.getFontMetrics().getStringBounds(s, g);
				g.drawString(s, Math.round((float)(offsetX + (i-minXValue)*xStep - rect.getCenterX())), 376);
			} else
				g.drawLine(offsetX + (i-minXValue)*xStep, 358, offsetX + (i-minXValue)*xStep, 362);
		}
		// draw y scale
		for (int i = (360 - yStep); i > 10; i -= yStep) {
			if (((i-360) % (yStep * 10)) == 0) {
				g.drawLine(160, i, 168, i);
				String s = Integer.toString((360 - i) / yStep);
				Rectangle2D rect = g.getFontMetrics().getStringBounds(s, g);
				g.drawString(s, new Double(158 - rect.getMaxX()).intValue(), new Double(i - rect.getCenterY()).intValue());
			} else {
				g.drawLine(162, i, 166, i);
			}
		}
		// calculate and draw the histograms
		maxValue = 20;
		for (int i = 0; (i < apList.size()) && (i < colorList.size()); i++) {
			int value = 0;
			int histogram[] = new int[maxXValue - minXValue + 1];
			
			String s = apList.get(i);
			for (int j = 0; j < histogram.length; j++)
				histogram[j] = 0;
			for (int j = 0; j < history.size(); j++) {
				Hashtable<String, Integer> historyEntry = history.get(j);
				if (historyEntry.containsKey(s)) {
					try {
						value = historyEntry.get(s) - minXValue;
						histogram[value]++;
					} catch (Exception ex) {}
				}
			}
			g.setColor(colorList.get(i));
			g.setFont(new Font(null, Font.PLAIN, 8));
			// draw the mac address and the rss count
			String macAddress = apList.get(i);
			g.drawString(macAddress + (showRSSCounts ? " (" + rssCount.get(macAddress) + ")" : ""), 10, 395 + (i * 13));
			// draw the meter bar
			if (showMeterBars)
				g.fillRect(offsetX, 385 + (i * 13), value * xStep, 8);
			// draw the histogram line
			for (int j = 1; j < histogram.length; j++) {
				if (histogram[j] > maxValue)
					maxValue = histogram[j];
				g.drawLine(offsetX + ((j-1) * xStep), 360 - ((histogram[j - 1]) * yStep),
				           offsetX + ( j    * xStep), 360 - ((histogram[j    ]) * yStep));
			}
		}
		if ((maxValue > oldMaxValue) || ((oldMaxValue - maxValue) > 10)) {
			yStep = new Double(Math.floor(350 / maxValue)).intValue();
			oldMaxValue = maxValue;
		}
	}
	
	public void addToHistory(Hashtable<String, Integer> reading) {
		Iterator<String> macIterator = reading.keySet().iterator();
		
		while (macIterator.hasNext()) {
			String macAddress = macIterator.next();
			if ((apList.size() < 16) && (!apList.contains(macAddress)))
				apList.add(macAddress);
			if (!rssCount.containsKey(macAddress))
				rssCount.put(macAddress, 1);
			else
				rssCount.put(macAddress, rssCount.get(macAddress) + 1);
		}

		// add the new reading to the history
		history.add(reading);
	}
	
	public void showMeterBars(boolean show) {
		if (show != showMeterBars) {
			showMeterBars = show;
		}
	}
	
	public void showRSSCounts(boolean show) {
		if (show != showRSSCounts) {
			showRSSCounts = show;
		}
	}
	
	public void resetHistogram() {
		history.clear();
		rssCount.clear();
		apList.clear();
	}
}
