/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace;

import java.io.PrintStream;
import java.util.Hashtable;

/**
 * Listener for a running trace operation.
 * 
 * @author Andreas
 */
public interface TraceListener {
	
	/**
	 * A trace has been initiated.
	 */
	public void traceInitiated();
	
	/**
	 * Add comments to trace file before trace entries.
	 * @param log	destination
	 */
	public void printPreamble(PrintStream log);
	
	/**
	 * Gets a preamble that precedes the line of trace results.
	 * @return	string of form a;b;c or a;b;c;
	 */
	public String getPreamble();
	
	/**
	 * Process trace result.
	 * @param readings	a map of AP MAC address to received signal strength
	 */
	public void traceResult(Hashtable<String, Integer> readings);
	
	/**
	 * A trace is finished. Add comments to trace file after trace entries.
	 */
	public void traceFinishing(PrintStream log);
	
	/**
	 * A trace has finished.
	 * @param success	whether the trace was successful
	 */
	public void traceFinished(boolean success);
	
}
