/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.util;

import java.util.HashMap;

/**
 * Counter
 * 
 * @author Andreas
 *
 * @param <T>	type of objects to be counted
 */
public class Counter<T> {
	protected HashMap<T, Integer> counts = new HashMap<T, Integer>();
	
	public Counter() {
	}
	
	/**
	 * Retrieves count for key.
	 * @param key	the key
	 * @return	the count
	 */
	public synchronized int getCount(T key) {
		if (counts.containsKey(key)) {
			return counts.get(key);
		} else return 0;
	}
	
	/**
	 * Increments the count for key.
	 * @param key	the key
	 */
	public synchronized void add(T key) {
		int oldValue = getCount(key);
		counts.put(key, oldValue + 1);
	}
	
	/**
	 * Returns the maximum count.
	 * 
	 * @return the count
	 */
	public synchronized int getMax() {
		int m = 0;
		for (T key: counts.keySet()) {
			int x = counts.get(key);
			if (x > m)
				m = x;
		}
		return m;
	}

}
