/*
 * This file is part of Loctrace.
 * 
 * Copyright (c) 2007 Thomas King <king@informatik.uni-mannheim.de>,
 * University of Mannheim, Germany
 * 
 * All rights reserved.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loctrace.util;

import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * Parser for position file.
 * 
 * @author Christian
 * @author Andreas
 */
public class PositionFileParser {
	
	/**
	 * Parses a position file.
	 * 
	 * @param file	the file
	 * @return	a list of positions
	 * @throws IOException
	 */
	public static List<Point2D> parse(File file) throws IOException {
		List<Point2D> positions = new ArrayList<Point2D>();
		BufferedReader in = new BufferedReader(new FileReader(file));
		String line;
		while ((line = in.readLine()) != null) {
			// copied from original code - no example file available
			line = line.replaceAll("degrees", "");
			line = line.replaceAll("^\\s+", "");
			line = line.replaceAll("\\s+$", "");
			line = line.replaceAll("[:,/()]", "");
			line = line.replaceAll("\\s+", ";");
			String[] parts = line.split(";");
			try {
				positions.add(new Point2D.Double(Double.parseDouble(parts[1]), Double.parseDouble(parts[2])));
			} catch (ArrayIndexOutOfBoundsException ex) {
				continue;
			} catch (NumberFormatException ex) {
				continue;
			}
		}
		// Sollte die Reihenfolge der Positionen nicht gerade durch die Datei vorgegeben sein?!
		// Hier wird sie überschrieben:
		Collections.sort(positions, new Comparator<Point2D>() {
			public int compare(Point2D a, Point2D b) {
				if (a.getX() < b.getX()) {
					return -1;
				} else if (a.getX() > b.getX()) {
					return 1;
				} else {
					if (a.getY() < b.getY()) {
						return -1;
					} else if (a.getY() > b.getY()) {
						return 1;
					} else return -1;
				}
			}
		});
		return positions;
	}
	
}
